@@ -1,5 +1,5 @@
 package ZMQ::Raw::Loop::Event;
-$ZMQ::Raw::Loop::Event::VERSION = '0.16';
+$ZMQ::Raw::Loop::Event::VERSION = '0.17';
 use strict;
 use warnings;
 use Carp;
@@ -14,7 +14,6 @@ BEGIN
 		read_handle
 		write_handle
 		loop
-		done
 		timeout
 		timer
 		on_set
@@ -39,7 +38,7 @@ ZMQ::Raw::Loop::Event - Event class
 
 =head1 VERSION
 
-version 0.16
+version 0.17
 
 =head1 DESCRIPTION
 
@@ -145,7 +144,7 @@ sub set
 	$this->write_handle->send ('');
 }
 
-=for Pod::Coverage done read_handle write_handle loop timeout timer on_set on_timeout
+=for Pod::Coverage read_handle write_handle loop timeout timer on_set on_timeout
 
 =head1 AUTHOR
 
