@@ -1,5 +1,5 @@
 package ZMQ::Raw::Loop::Handle;
-$ZMQ::Raw::Loop::Handle::VERSION = '0.16';
+$ZMQ::Raw::Loop::Handle::VERSION = '0.17';
 use strict;
 use warnings;
 use Carp;
@@ -12,7 +12,6 @@ BEGIN
 {
 	@attributes = qw/
 		handle
-		done
 		timer
 		loop
 		timeout
@@ -39,7 +38,7 @@ ZMQ::Raw::Loop::Handle - Handle class
 
 =head1 VERSION
 
-version 0.16
+version 0.17
 
 =head1 DESCRIPTION
 
@@ -127,7 +126,7 @@ sub new
 	return bless $self, $class;
 }
 
-=for Pod::Coverage done handle timer loop timeout on_readable on_writable on_timeout
+=for Pod::Coverage handle timer loop timeout on_readable on_writable on_timeout
 
 =head1 AUTHOR
 
