@@ -1,5 +1,5 @@
 package ZMQ::Raw::Loop::Timer;
-$ZMQ::Raw::Loop::Timer::VERSION = '0.16';
+$ZMQ::Raw::Loop::Timer::VERSION = '0.17';
 use strict;
 use warnings;
 use Carp;
@@ -11,7 +11,6 @@ my @attributes;
 BEGIN
 {
 	@attributes = qw/
-		done
 		timer
 		loop
 		on_timeout
@@ -35,7 +34,7 @@ ZMQ::Raw::Loop::Timer - Timer class
 
 =head1 VERSION
 
-version 0.16
+version 0.17
 
 =head1 DESCRIPTION
 
@@ -101,13 +100,22 @@ sub new
 	return bless $self, $class;
 }
 
+
+
 sub cancel
 {
 	my ($this) = @_;
+
 	$this->timer->cancel;
+
+	if ($this->loop)
+	{
+		$this->loop->remove ($this);
+		$this->loop (undef);
+	}
 }
 
-=for Pod::Coverage done timer loop on_timeout
+=for Pod::Coverage timer loop on_timeout
 
 =head1 AUTHOR
 
